import json

def printResults(listResults, what):
	if len(listResults):
		print '\nResults on ' + what + ' are: \n--------------------------------------------\n'

	for i in range(0, len(listResults)):
		ob = (listResults[i])['_source']

		print '*  File Name: ' + str(ob['file'])
		print '*  Description: ' + str(ob['description'])
		print '*  File Path: ' + str(ob['path'])
		print '*  File Tags: ' + str(ob['tags'])
		
		if i != len(listResults)-1:
			print '\n'		


def query(es, what):

	if what == 'tags':
		tags = raw_input("Enter tags to be searched [space separated]: ")
		doc = {
			"query": {
				"query_string": {
					"query": tags,
					"fields": ["tags"]
				}
			}
		}

		res = es.search(index='tagz', doc_type='files', body=doc)
		lst = (res['hits'])['hits']

		printResults(lst, what)

	elif what == 'name':
		name = raw_input("File name to be searched [with extension]: ")
		doc = {
			"query": {
				"query_string": {
					"query": name,
					"fields": ["file"]
				}
			}
		}

		res = es.search(index='tagz', doc_type='files', body=doc)
		lst = (res['hits'])['hits']

		printResults(lst, what)

	elif what == 'description':
		desc = raw_input("Rough description to be searched : ")
		doc = {
			"query": {
				"query_string": {
					"query": desc,
					"fields": ["description"]
				}
			}
		}

		res = es.search(index='tagz', doc_type='files', body=doc)
		lst = (res['hits'])['hits']

		printResults(lst, what)		



def duplicate(es, idx, otype, doc):
	path = doc['path']

	doc = {
		"query" : {
			"filtered" : { 
				"query" : {
					"match_all" : {} 
				},
				"filter" : {
					"term" : { 
						"path": path
					}
				}
			}
		}
	}

	res = es.search(index='tagz', doc_type='files', body=doc)
	lst = (res['hits'])['hits']

	if len(lst):
		return (True, (lst[0])['_id'])

	return False


def indexDoc(es, idx, otype, oid, doc):
	try:
		ret = duplicate(es, idx, otype, doc)

		if ret == False:
			res = es.index(index=idx, doc_type=otype, id=oid, body=doc)
			print 'Okay!! Indexed'

		else:
			resp = raw_input('This file is already indexed. Do you want to use the recent tags/description for this file. ? (y/n): ')

			if resp == 'y':
				res = es.index(index=idx, doc_type=otype, id=ret[1], body=doc)
				print 'Okay!! Indexed'

			else:
				return

	except:
		print 'Encountered error while indexing. Is ElasticSearch instance running ??'


# Experimental File

import argparse
import os, json
from elasticsearch import Elasticsearch
import uuid
from utils.querying import query
from utils.indexing import indexDoc

def main():
	parser = argparse.ArgumentParser()
	es = Elasticsearch()

	# Currently only optional argument support (later add optional arguments using subparsers)
	parser.add_argument("-f", "--filename", help="you want to index a new file")
	parser.add_argument("-st", "--searchtags", help="search for a file based on tags", action="store_true")
	parser.add_argument("-sd", "--searchdescription", help="search for a file based on its description", action="store_true")
	parser.add_argument("-sn", "--searchname", help="search for a file based on its name", action="store_true")

	args = parser.parse_args()

	if args.filename:
		name = args.filename

		if os.path.isfile(name) == False:
			print 'Needed a file'
			return

		tags = (raw_input("Tags [separated by a space]: ")).split(' ')
		desc = (raw_input("Description [short description]: "))
		path = os.getcwd() + '/' + args.filename

		doc = {
			"file": name,
			"path": path,
			"tags": tags,
			"description": desc
		}

		unique_id = str(uuid.uuid4())
		indexDoc(es, 'tagz', 'files', unique_id, doc)

	if args.searchtags:
		query(es, 'tags')

	if args.searchdescription:
		query(es, 'description')

	if args.searchname:
		query(es, 'name')


if __name__ == '__main__':
	main()



from setuptools import setup
from setuptools import find_packages

setup(
	name = "ktagz",
	packages = find_packages(),
	version = "0.1.0",
	description = "A command line file-tagger/file-search tool for linux",
	long_description = open('README.md').read(),
	author = "Khirod Kant Naik",
	author_email = "khirod234@gmail.com",
	url='https://github.com/shinigamiryuk/TagZ',
	scripts = ['bin/ktagz', 'bin/createIndex.sh'],
	install_requires=[
	"elasticsearch >= 1.0.0",
	]
)
